# Two stage example Virtual Machine file
# moves get set in Main
# usb port needs to be set in initInterfaces
# Nadya Peek Dec 2014

#------IMPORTS-------
from pygestalt import nodes
from pygestalt import interfaces
from pygestalt import machines
from pygestalt import functions
from pygestalt.machines import elements
from pygestalt.machines import kinematics
from pygestalt.machines import state
from pygestalt.utilities import notice
from pygestalt.publish import rpc	#remote procedure call dispatcher
import time
import io
import sys
from PyQt5 import QtWidgets
from Tkinter import Tk
from tkFileDialog import askopenfilename
import numpy as np
import cv2
import scipy.misc
import os

from PyQt5.QtWidgets import QLabel
from PIL import Image
from PyQt5 import QtWidgets, QtGui, QtCore

from scipy.ndimage import imread
class Window(QtWidgets.QWidget):

    def __init__(self):
        super(Window,self).__init__()

        self.init_ui()

    def init_ui(self):
    	self.b0 = QtWidgets.QPushButton('Load Image')
        self.b = QtWidgets.QPushButton('Felting Machine')
        self.l = QtWidgets.QLabel('Filename')
        self.label = QLabel(self)

        h_box = QtWidgets.QHBoxLayout()
        h_box.addStretch()
        h_box.addWidget(self.l)
        h_box.addStretch()

        v_box = QtWidgets.QVBoxLayout()
        v_box.addWidget(self.b0)
        v_box.addWidget(self.b)
        v_box.addLayout(h_box)
        v_box.addWidget(self.label)


        self.setLayout(v_box)
        self.setWindowTitle('Felting Machine')
        self.b0.clicked.connect(self.btn0_click)
        self.b.clicked.connect(self.btn_click)
        self.stages = virtualMachine(persistenceFile = "test.vmp")
        self.show()
        
    def btn0_click(self):
		Tk().withdraw()
		filename = askopenfilename()
		self.T=8   # tool diameter
		self.D=48  # Size of a side of fabric
		self.O=2   # overlap
	
		self.img = cv2.imread(filename,0)
		self.l.setText(filename)

		height, width = self.img.shape[:2]
		self.reduc=self.D*self.O/float(self.T*height)  # (D/(T/O))/height
		self.dst=scipy.misc.imresize(self.img, self.reduc)
		img2 = Image.fromarray(self.dst )
		img2.save('my.png')
		img2.show()
        
# 		pixmap01 = QtGui.QPixmap.fromImage(img2)
# 		pixmap_image = QtGui.QPixmap(pixmap01)
# 		self.label.setPixmap(pixmap_image)
# 		self.label.show()

    def btn_click(self):
        
        heightd, widthd = self.dst.shape[:2]
        movesxyz = [[0,0,0]]
        sp=self.T*2;
        for h in range(0,heightd-1,sp):          
        	movesxyz.append([h,0,0])
        	movesxyz.append([h,0,1])
        	movesxyz.append([h,0,0])

        for w in range(0,widthd-1,sp):
        	movesxyz.append([heightd-1,w,0])
        	movesxyz.append([heightd-1,w,1])
        	movesxyz.append([heightd-1,w,0])

        for h in range(heightd-1,0,-sp):
        	movesxyz.append([h,widthd-1,0])
        	movesxyz.append([h,widthd-1,1])
        	movesxyz.append([h,widthd-1,0])

        for w in range(widthd-1,0,-sp):
        	movesxyz.append([0,w,0])
        	movesxyz.append([0,w,1])
        	movesxyz.append([0,w,0])

        for h in range(0,heightd-1):
         	if h%2==0:
         		for w in range(0,widthd-1):
 	         		if self.dst[h,w]!=0:
 	         			movesxyz.append([h,w,0])
 	         			movesxyz.append([h,w,1])
 	         			movesxyz.append([h,w,0])

 	        else:
 	         	for w in range(widthd-1,0,-1):
 	         		if self.dst[h,w]!=0:
 	         			movesxyz.append([h,w,0])
 	         			movesxyz.append([h,w,1])
 	         			movesxyz.append([h,w,0])
 		
        movesxyz.append([0,0,0])
        print movesxyz
        
        self.stages.xyzNode.setVelocityRequest(8)	
        #moves = [[0,0],[-10,-10],[10,10],[30,30],[20,20],[0,0]]
        for move in movesxyz:
        	self.stages.move(move, 0)
        	status = self.stages.xAxisNode.spinStatusRequest()
        	# This checks to see if the move is done.
        	while status['stepsRemaining'] > 0:
        		time.sleep(0.001)
        		status = self.stages.xAxisNode.spinStatusRequest()	
	


#------VIRTUAL MACHINE------
class virtualMachine(machines.virtualMachine):
	
	def initInterfaces(self):
		if self.providedInterface: self.fabnet = self.providedInterface		#providedInterface is defined in the virtualMachine class.
		else: self.fabnet = interfaces.gestaltInterface('FABNET', interfaces.serialInterface(baudRate = 115200, interfaceType = 'ftdi', portName = 'COM3'))
  		
	def initControllers(self):
		self.xAxisNode = nodes.networkedGestaltNode('X Axis', self.fabnet, filename = '086-005a.py', persistence = self.persistence)
		self.yAxisNode = nodes.networkedGestaltNode('Y Axis', self.fabnet, filename = '086-005a.py', persistence = self.persistence)
		self.zAxisNode = nodes.networkedGestaltNode('z Axis', self.fabnet, filename = '086-005a.py', persistence = self.persistence)
  
		self.xyzNode = nodes.compoundNode(self.xAxisNode, self.yAxisNode, self.zAxisNode)
  
	def initCoordinates(self):
		self.position = state.coordinate(['mm', 'mm', 'mm'])
  	
	def initKinematics(self):
		self.xAxis = elements.elementChain.forward([elements.microstep.forward(4), elements.stepper.forward(1.8), elements.leadscrew.forward(1), elements.invert.forward(True)])
		self.yAxis = elements.elementChain.forward([elements.microstep.forward(4), elements.stepper.forward(1.8), elements.leadscrew.forward(1), elements.invert.forward(False)])
		self.zAxis = elements.elementChain.forward([elements.microstep.forward(4), elements.stepper.forward(1.8), elements.leadscrew.forward(1), elements.invert.forward(False)])		
  		
		self.stageKinematics = kinematics.direct(3)	#direct drive on all axes
  	
	def initFunctions(self):
		self.move = functions.move(virtualMachine = self, virtualNode = self.xyzNode, axes = [self.xAxis, self.yAxis, self.zAxis], kinematics = self.stageKinematics, machinePosition = self.position,planner = 'null')
		self.jog = functions.jog(self.move)	#an incremental wrapper for the move function
		pass
  		
	def initLast(self):
		#self.machineControl.setMotorCurrents(aCurrent = 0.8, bCurrent = 0.8, cCurrent = 0.8)
		#self.xNode.setVelocityRequest(0)	#clear velocity on nodes. Eventually this will be put in the motion planner on initialization to match state.
		pass
  	
	def publish(self):
		#self.publisher.addNodes(self.machineControl)
		pass
  	
	def getPosition(self):
		return {'position':self.position.future()}
  	
	def setPosition(self, position  = [None]):
		self.position.future.set(position)
  
 	def setSpindleSpeed(self, speedFraction):
		self.machineControl.pwmRequest(speedFraction)
		pass

#------IF RUN DIRECTLY FROM TERMINAL------
if __name__ == '__main__':
# 	app = QtWidgets.QApplication(sys.argv)
# 	window =Windowx()
#   	sys.exit(app.exec_())
	
	app = QtWidgets.QApplication(sys.argv)
	a_window = Window()
	sys.exit(app.exec_())
	# The persistence file remembers the node you set. It'll generate the first time you run the
	# file. If you are hooking up a new node, delete the previous persistence file.



